require 'xaases/js/function'
module Xaases
  class Js
    attr_reader :minify

    def initialize(minify: true)
      @minify = minify
      @s = ''
    end

    def strict!
      if minify
        @s = "'use strict';"
      else
        @s = "'use strict';\n"
      end
    end

    def const(list)
      if minify
        @s += 'const ' + list.map do |key, value|
          key.to_s + '=' + value.to_s
        end.join(',') + ';'
      else
        @s += 'const ' + list.map do |key, value|
          key.to_s + ' = ' + value.to_s
        end.join(",\n      ") + ";\n"
      end
    end

    def call(name, *params)
      if minify
        @s += "#{name}(#{params.join(',')});"
      else
        @s += "#{name}(#{params.join(', ')});"
      end
    end

    def hash(data)
      if minify
        @s += '{' + data.map do |key, value|
          key.to_s + ':' + value.to_s
        end.join(',') + '}'
      else
        @s += '{ ' + data.map do |key, value|
          key.to_s + ': ' + value.to_s
        end.join(', ') + ' }'
      end
    end

    def if(*conditions)
      i = -1
      conditions.map do |condition|
        i += 1
        if i == 0
          'if{' + condition + '}'
        elsif i == conditions.length - 1
          'else{' + condition + '}'
        else
          'elsif{' + condition + '}'
        end
      end.join
    end

    def export(name, f)
      if minify
        @s += "module.exports.#{name}=(#{f.params.join(',')})=>{#{f.render}};"
      else
        content = f.render.split("\n").join("\n  ")
        @s += "module.exports.#{name} = (#{f.params.join(', ')}) => {\n  #{content}\n};\n"
      end
    end

    def render
      @s
    end
  end
end
