require 'yaml'

module Xaases
  module Serverless
    class Yaml
      attr_writer :data

      def initialize(filename, name = nil)
        @filename = filename
        begin
          @data = YAML.load_file filename
        rescue Errno::ENOENT
          @data = {
            'service' => name,
            'provider' => {},
            'resources' => {'Resources' => {}},
            'functions' => {}
          }
        end
      end

      def provider=(prov)
        @data['provider'] = prov
      end

      def add_resource(name, rs)
        @data['resources']['Resources'][name] = rs
      end

      def add_function(method, path)
        method = method.downcase
        file = 'index'
        name = path.sub(/\A\//, '').sub(/\//, '_')
        name = nil if name == ''

        name = ['create', name].compact.join('_') if method == 'post'
        name = ['update', name].compact.join('_') if method == 'put'
        name = ['delete', name].compact.join('_') if method == 'delete'

        name = 'root' if method == 'get' && !name
        @data['functions'][name] = {
          'handler' => "handler.#{name}",
          'events' => [{ 'http' => { 'path' => path, 'method' => method }}]
        }
        name
      end

      def write
        YAML.dump @data, File.open(@filename, 'w')
      end
    end
  end
end
